//!OpenSCAD

Height = 50;
Width = 100;
Depth = 150;
Thickness = 5;
Part_Spacing = 15;
Rounding = false;
Endmill_Diameter = 6.35;
Generate_DXF = true;
Intersection = true;
Detail = 40;

$fn = Detail * 1;

//Height = 3;
//Width = 4;
//Depth = 5;
//Thickness = 0.125;
//Part_Spacing = 0.5;
//Generate_DXF = true;
//Endmill_Diameter = 0.25;
//Rounding = false;

endmillradius = Endmill_Diameter / 2;
widthnum = floor((Width / Endmill_Diameter) / 2) * 2 - 1;
widthinsetnum = floor(((Width - Thickness * 2) / Endmill_Diameter) / 2) * 2 - 1;
depthnum = floor(((Depth - Thickness * 2) / Endmill_Diameter) / 2) * 2 - 1;
heightnum = floor(((Height - Thickness * 2) / Endmill_Diameter) / 2) * 2 - 1;
fingerXwidth = Width / widthnum;
fingerinsetXwidth = (Width - Thickness * 2) / widthinsetnum;
fingerYwidth = Depth / depthnum;
fingerinsetYwidth = (Depth - Thickness * 2) / depthnum;
fingerZwidth = (Height - Thickness * 2) / heightnum;

tool_radius_tip = 0.79375;
tool_radius_width = 3.175;

module radiuscut(beginx, beginy, endx, endy) {
n = Detail;
step = 360/n;
    
hull(){
translate([beginx,beginy,0])
cylinder(step,tool_radius_tip,tool_radius_tip);
translate([endx,endy,0])
cylinder(step,tool_radius_tip,tool_radius_tip);
}

hull(){
translate([beginx,beginy,tool_radius_width])
cylinder(tool_radius_width*2,tool_radius_tip+tool_radius_width,tool_radius_tip+tool_radius_width);
    
translate([endx,endy,tool_radius_width])
cylinder(tool_radius_width*2,tool_radius_tip+tool_radius_width,tool_radius_tip+tool_radius_width);
}

for (i=[0:step:90]) {
    angle = i;
    dx = tool_radius_width*cos(angle-step);
    dxx = tool_radius_width*cos(angle);
    dzz = tool_radius_width*sin(angle-step);
    dz = tool_radius_width*sin(angle);
    dh = dz-dzz;
    hull(){
    translate([beginx,beginy,dz])
       cylinder(dh,tool_radius_tip+tool_radius_width-dx,tool_radius_tip+tool_radius_width-dxx);
    translate([endx,endy,dz])
       cylinder(dh,tool_radius_tip+tool_radius_width-dx,tool_radius_tip+tool_radius_width-dxx);
        }
}
}

module board(ht, wd, dpth) {
  cube([wd, dpth, ht], center=false);
}

module fingercuts(fw, fd, axis, num, len, evenorodd, offset, relieveends) {
  union(){
    if (relieveends > 0 && offset == 0) {
      if (axis == "x") {
        union(){
          translate([(-endmillradius), 0, 0]){
            fingercut(relieveends * 1.01 + endmillradius, fd);
          }
          translate([(len + (relieveends - relieveends * 0.01)), 0, 0]){
            fingercut(relieveends * 1.01 + endmillradius, fd);
          }
        }
      } else {
        union(){
          translate([0, (-endmillradius), 0]){
            fingercut(relieveends, fd);
          }
          translate([0, (len + relieveends), 0]){
            fingercut(relieveends, fd);
          }
        }
      }

    } else if (relieveends == 0 && offset > 0) {
      if (axis == "x") {
        union(){
          translate([(-offset), 0, 0]){
            fingercut(offset * 3, fd);
          }
          translate([(len - offset * 2), relieveends, 0]){
            fingercut(offset * 3, fd);
          }
        }
      } else {
        if (axis == "y") {
          union(){
            translate([0, (-offset), 0]){
              fingercut(fd, offset + fw);
            }
            translate([0, (len - fw), 0]){
              fingercut(fd, offset + fw);
            }
          }
        }

      }

    } else if (relieveends > 0 && offset > 0) {
      if (axis == "x") {
        if (evenorodd == "even") {
          union(){
            translate([(-offset), 0, 0]){
              fingercut(offset + relieveends, fd);
            }
            translate([(len - relieveends), 0, 0]){
              fingercut(offset + relieveends, fd);
            }
          }
        } else {
          union(){
            translate([(-offset), 0, 0]){
              fingercut((offset + relieveends) + fw, fd);
            }
            translate([(len - (relieveends + fw)), 0, 0]){
              fingercut((offset + relieveends) + fw, fd);
            }
          }
        }

      } else {
        if (axis == "y") {
          union(){
            translate([0, (-offset), 0]){
              fingercut(offset + relieveends, fd);
            }
            translate([0, (len - relieveends), 0]){
              fingercut(offset + relieveends, fd);
            }
          }
        }

      }

    }

    if (evenorodd == "odd" && axis == "x") {
      if (relieveends > 0) {
        translate([relieveends, 0, 0]){
          fingerX(1, num, fw, fd);
        }
      } else {
        fingerX(1, num, fw, fd);
      }

    }

    if (evenorodd == "even" && axis == "y") {
      if (relieveends > 0) {
        translate([0, relieveends, 0]){
          fingerY(2, num, fw, fd);
        }
      } else {
        fingerY(2, num, fw, fd);
      }

    }

    if (evenorodd == "even" && axis == "x") {
      if (relieveends > 0) {
        translate([relieveends, 0, 0]){
          fingerX(2, num, fw, fd);
        }
      } else {
        fingerX(2, num, fw, fd);
      }

    }

    if (evenorodd == "odd" && axis == "y") {
      if (relieveends > 0) {
        translate([0, relieveends, 0]){
          fingerY(1, num, fw, fd);
        }
      } else {
        fingerY(1, num, fw, fd);
      }

    }

  }
}

module frontback() {
  difference() {
    board(Thickness, Width, Height);

    union(){
      translate([0, (-endmillradius), 0]){
        fingercuts(fingerXwidth, Thickness + endmillradius, "x", widthnum, Width, "odd", endmillradius, 0);
      }
      translate([0, (Height - Thickness), 0]){
        fingercuts(fingerXwidth, Thickness + endmillradius, "x", widthnum, Width, "odd", endmillradius, 0);
      }
      translate([(-endmillradius), 0, 0]){
        fingercuts(fingerZwidth, Thickness + endmillradius, "y", heightnum, Height, "even", endmillradius, Thickness);
      }
      translate([(Width - Thickness), 0, 0]){
        fingercuts(fingerZwidth, Thickness + endmillradius, "y", heightnum, Height, "even", endmillradius, Thickness);
      }
          
if (Rounding == true) {
    for (i=[1:2:widthnum-1]){
translate([0,-tool_radius_tip,Thickness-tool_radius_width]){
translate([i*fingerXwidth-tool_radius_tip,0,0]){
radiuscut(0, 0, 0, Thickness-tool_radius_tip);
radiuscut(fingerXwidth+tool_radius_tip*2, 0, fingerXwidth+tool_radius_tip*2, Thickness-tool_radius_tip);
}}}
translate([0,Height+tool_radius_tip*2-Thickness,0]){
    for (i=[1:2:widthnum-1]){
translate([0,0,Thickness-tool_radius_width]){
translate([i*fingerXwidth-tool_radius_tip,0,0]){
radiuscut(0, 0, 0, Thickness-tool_radius_tip);
radiuscut(fingerXwidth+tool_radius_tip*2, 0, fingerXwidth+tool_radius_tip*2, Thickness-tool_radius_tip);
}}}
}

    for (i=[1:2:heightnum-1]){
translate([-tool_radius_tip,Thickness,Thickness-tool_radius_width]){
translate([0,i*fingerZwidth+tool_radius_tip,0]){
radiuscut(0, 0, Thickness-tool_radius_tip,0);
translate([0,fingerZwidth-tool_radius_tip*2,0]){radiuscut(0, 0, Thickness-tool_radius_tip,0);
}
}
translate([Width-tool_radius_tip*2,0,0])
{translate([0,i*fingerZwidth+tool_radius_tip,0]){
radiuscut(0, 0, Thickness-tool_radius_tip,0);
translate([0,fingerZwidth-tool_radius_tip*2,0]){radiuscut(0, 0, Thickness-tool_radius_tip,0);
}}
}
}
}


}    

 

}
  }
}

module sides() {
  difference() {
    board(Thickness, Height, Depth);

    union(){
      translate([0, (-endmillradius), 0]){
        fingercuts(fingerZwidth, Thickness + endmillradius, "x", heightnum, Height, "odd", endmillradius, Thickness);
      }
      translate([0, (Depth - Thickness), 0]){
        fingercuts(fingerZwidth, Thickness + endmillradius, "x", heightnum, Height, "odd", endmillradius, Thickness);
      }
      translate([(-endmillradius), 0, 0]){
        fingercuts(fingerYwidth, Thickness + endmillradius, "y", depthnum, Depth, "odd", endmillradius, 0);
      }
      translate([(Height - Thickness), 0, 0]){
        fingercuts(fingerYwidth, Thickness + endmillradius, "y", depthnum, Depth, "odd", endmillradius, 0);
      }
    }
if (Rounding == true) {
    for (i=[1:2:heightnum-1]){
translate([Thickness,-tool_radius_tip,Thickness-tool_radius_width]){
translate([i*fingerZwidth-tool_radius_tip,0,0]){
radiuscut(0, 0, 0, Thickness-tool_radius_tip);
radiuscut(fingerZwidth+tool_radius_tip*2, 0, fingerZwidth+tool_radius_tip*2, Thickness-tool_radius_tip);
}}}
translate([0,Depth+endmillradius-Thickness,0])
{
    for (i=[1:2:widthnum-1]){
translate([Thickness,0,Thickness-tool_radius_width]){
translate([i*fingerZwidth-tool_radius_tip*2,0,0]){
radiuscut(0, 0, 0, Thickness-tool_radius_tip);
radiuscut(fingerZwidth+tool_radius_tip, 0, fingerZwidth+tool_radius_tip*2, Thickness-tool_radius_tip);
}}}
}

    for (i=[1:2:depthnum-1]){
translate([-tool_radius_tip,0,Thickness-tool_radius_width]){
translate([0,i*fingerYwidth-tool_radius_tip,0]){
radiuscut(0, 0, Thickness-tool_radius_tip,0);
translate([0,fingerYwidth+tool_radius_tip*2,0]){radiuscut(0, 0, Thickness-tool_radius_tip,0);
}
}
translate([Height-tool_radius_tip*2,0,0])
{translate([0,i*fingerYwidth-tool_radius_tip,0]){
radiuscut(0, 0, Thickness-tool_radius_tip,0);
translate([0,fingerYwidth+tool_radius_tip*2,0]){radiuscut(0, 0, Thickness-tool_radius_tip,0);
}}
}
}
}

}

  }
}

module topbottom() {
  difference() {
    board(Thickness, Width, Depth);

    translate([0, (-endmillradius), 0]){
      fingercuts(fingerXwidth, Thickness + endmillradius, "x", widthnum, Width - Thickness * 2, "even", 0, 0);
    }
    translate([0, (Depth - Thickness), 0]){
      fingercuts(fingerXwidth, Thickness + endmillradius, "x", widthnum, Width - Thickness * 2, "even", 0, 0);
    }
    translate([(-endmillradius), 0, 0]){
      fingercuts(fingerYwidth, Thickness + endmillradius, "y", depthnum, Depth - Thickness * 2, "even", 0, 0);
    }
    translate([(Width - Thickness), 0, 0]){
      fingercuts(fingerYwidth, Thickness + endmillradius, "y", depthnum, Depth - Thickness * 2, "even", 0, 0);
    }
if (Rounding == true) {
    for (i=[1:2:widthnum-1]){
translate([0,-tool_radius_tip,Thickness-tool_radius_width]){
translate([i*fingerXwidth+tool_radius_tip,0,0]){
radiuscut(0, 0, 0, Thickness-tool_radius_tip);
radiuscut(fingerXwidth-tool_radius_tip, 0, fingerXwidth-tool_radius_tip, Thickness-tool_radius_tip);
}}}
translate([0,Depth+endmillradius-Thickness,0])
{
    for (i=[1:2:widthnum-1]){
translate([0,0,Thickness-tool_radius_width]){
translate([i*fingerXwidth+tool_radius_tip,0,0]){
radiuscut(0, 0, 0, Thickness-tool_radius_tip);
radiuscut(fingerXwidth-tool_radius_tip, 0, fingerXwidth-tool_radius_tip, Thickness-tool_radius_tip);
}}}
}
}
  }
}

module fingercut(fingerwidth, fingerdepth) {
  if (Rounding == false) {
    translate([0, 0, (-(Thickness / 2))]){
      cube([fingerwidth, fingerdepth, (Thickness * 2)], center=false);
    }
  } else {
    hull(){
      translate([endmillradius, endmillradius, (-(Thickness / 2))]){
        cylinder(r1=endmillradius, r2=endmillradius, h=(Thickness * 2), center=false);
      }
      translate([(fingerwidth - endmillradius), (fingerdepth - endmillradius), (-(Thickness / 2))]){
        cylinder(r1=endmillradius, r2=endmillradius, h=(Thickness * 2), center=false);
      }
      translate([endmillradius, (fingerdepth - endmillradius), (-(Thickness / 2))]){
        cylinder(r1=endmillradius, r2=endmillradius, h=(Thickness * 2), center=false);
      }
      translate([(fingerwidth - endmillradius), endmillradius, (-(Thickness / 2))]){
        cylinder(r1=endmillradius, r2=endmillradius, h=(Thickness * 2), center=false);
      }
    }
  }

}

module fingerX(beg, numb, ffw, ffd) {
  for (i = [beg : abs(2) : numb]) {
    translate([((i - 1) * ffw), 0, 0]){
      fingercut(ffw, ffd);
    }
  }

}

module fingerY(beg, numb, ffw, ffd) {
  for (j = [beg : abs(2) : numb]) {
    translate([0, ((j - 1) * ffw), 0]){
      fingercut(ffd, ffw);
    }
  }

}

if (Generate_DXF == true) {
  union(){
    translate([0, (Height + Part_Spacing), 0]){
      sides();
    }
    translate([(Height + Part_Spacing), 0, 0]){
      frontback();
    }
    translate([(Height + Part_Spacing), (Height + Part_Spacing), 0]){
      topbottom();
    }
    translate([(Height + Part_Spacing), ((Height + Part_Spacing) + (Depth + Part_Spacing)), 0]){
      frontback();
    }
    translate([((Height + Part_Spacing) + (Width + Part_Spacing)), (Height + Part_Spacing), 0]){
      sides();
    }
  }
} else {
if (Intersection == true) {
  intersection(){
      topbottom();
    translate([0, 0, Height]){
      rotate([270, 0, 0]){
        frontback();
      }
    }
}
} else {
  union(){
    translate([0, Part_Spacing, (Height + Part_Spacing)]){
      rotate([0, 90, 0]){
        sides();
      }
    }
    translate([Part_Spacing, 0, Height + Part_Spacing]){
      rotate([270, 0, 0]){
        frontback();
      }
    }
    translate([Part_Spacing, Part_Spacing, 0]){
      topbottom();
    }
    translate([Width+Part_Spacing, Part_Spacing, Height + Part_Spacing]){
rotate([0, 180, 0]){topbottom();}
    }
    translate([(Width + Part_Spacing * 2), Part_Spacing, Part_Spacing]){
      rotate([0, 270, 0]){
        sides();
      }
    }
    translate([Part_Spacing, (Depth + Part_Spacing * 2), Part_Spacing]){
      rotate([90, 0, 0]){
        frontback();
      }
    }
  }
}
}

